import React from 'react';
import TodoModel from './TodoModel';
import TodoItem from './TodoItem';
import TodoFooter from './TodoFooter';

const ENTER_KEY = 13;

class App extends React.Component {
  handleNewTodoKeyDown(event) {
    if (event.keyCode !== ENTER_KEY) {
      return;
    }

    event.preventDefault();


    var val = event.target.value.trim();

    if (val) {
      this.props.model.addTodo(val);
      React.findDOMNode(this.refs.newField).value = '';
    }
  };

  toggle(todoToToggle) {
    this.props.model.toggle(todoToToggle);
  };

  destroy(todo) {
    this.props.model.destroy(todo);
  };

  save(todoToSave, text) {
    this.props.model.save(todoToSave, text);
  };

  render() {
    let footer,
      main,
      todos = this.props.model.todos;

    var todoItems = todos.map(function (todo) {
      return (
        <TodoItem
          key={todo.id}
          todo={todo}
          onToggle={this.toggle.bind(this, todo)}
          onDestroy={this.destroy.bind(this, todo)}
          onSave={this.save.bind(this, todo)}
          onCancel={this.cancel} />
      );
    }, this);

    var activeTodoCount = todos.reduce(function (accum, todo) {
      return todo.completed ? accum : accum + 1;
    }, 0);

    var completedCount = todos.length - activeTodoCount;

    if (activeTodoCount || completedCount) {
      footer =
        <TodoFooter
          count={activeTodoCount}
          completedCount={completedCount}
          onClearCompleted={this.clearCompleted} />;
    }

    if (todos.length) {
      main = (
        <section>
          <div className="checkbox">
            <label>
              <input
                type="checkbox"
                onChange={this.toggleAll}
                checked={activeTodoCount === 0} /> Toggle all
            </label>
          </div>
          <ul className="todo-list">
            {todoItems}
          </ul>
        </section>
      );
    }

    return (
      <div>
        <header className="header">
          <h1>todos</h1>
          <div className="form-group">
            <input
              ref="newField"
              className="form-control"
              placeholder="What needs to be done?"
              onKeyDown={this.handleNewTodoKeyDown.bind(this)}
              autoFocus={true} />
          </div>
        </header>
        {main}
        {footer}
      </div>
    );
  }
}

var model = new TodoModel('react-todos');

const rerenderApp = function () {
  React.render(<App model={model} />, document.getElementById('todoapp'));
}

rerenderApp();
model.subscribe(rerenderApp);
